@@ -79,20 +79,6 @@ defmodule Ejabberd.MixProject do
     end
   end
 
-  defp if_type_exported(module, typeDef, okResult) do
-    try do
-      {:ok, concrete} = :dialyzer_utils.get_core_from_beam(:code.which(module))
-      {:ok, types} = :dialyzer_utils.get_record_and_type_info(concrete)
-      if Map.has_key?(types, typeDef) do
-        okResult
-      else
-        []
-      end
-    rescue
-       _ -> []
-    end
-  end
-
   defp erlc_options do
     # Use our own includes + includes from all dependencies
     includes = ["include", deps_include()]
@@ -113,7 +99,7 @@ defmodule Ejabberd.MixProject do
              if_version_below(~c"25", [{:d, :OTP_BELOW_25}]) ++
              if_version_below(~c"26", [{:d, :OTP_BELOW_26}]) ++
              if_version_below(~c"27", [{:d, :OTP_BELOW_27}]) ++
-             if_type_exported(:odbc, {:opaque, :connection_reference, 0}, [{:d, :ODBC_HAS_TYPES}])
+             if_version_below(~c"28", [{:d, :OTP_BELOW_28}])
     defines = for {:d, value} <- result, do: {:d, value}
     result ++ [{:d, :ALL_DEFS, defines}]
   end
@@ -133,9 +119,9 @@ defmodule Ejabberd.MixProject do
     [{:cache_tab, "~> 1.0"},
      {:dialyxir, "~> 1.2", only: [:test], runtime: false},
      {:eimp, "~> 1.0"},
-     {:ex_doc, "~> 0.31", only: [:dev, :edoc], runtime: false},
+     {:ex_doc, "~> 0.31", only: [:edoc], runtime: false},
      {:fast_tls, "~> 1.1.22"},
-     {:fast_xml, "~> 1.1.53", override: true},
+     {:fast_xml, "~> 1.1.53"},
      {:fast_yaml, "~> 1.0"},
      {:idna, "~> 6.0"},
      {:mqtree, "~> 1.0"},
@@ -144,8 +130,8 @@ defmodule Ejabberd.MixProject do
      {:p1_utils, "~> 1.0"},
      {:pkix, "~> 1.0"},
      {:stringprep, ">= 1.0.26"},
-     {:xmpp, "~> 1.9.1"},
-     {:yconf, "~> 1.0.17"}]
+     {:xmpp, "~> 1.10.0"},
+     {:yconf, ">= 1.0.18"}]
     ++ cond_deps()
   end
 
@@ -175,7 +161,7 @@ defmodule Ejabberd.MixProject do
                          {if_version_below(~c"22", true), {:lager, "~> 3.9.1"}},
                          {config(:lua), {:luerl, "~> 1.2.0"}},
                          {config(:mysql), {:p1_mysql, ">= 1.0.24"}},
-                         {config(:pgsql), {:p1_pgsql, ">= 1.1.26"}},
+                         {config(:pgsql), {:p1_pgsql, ">= 1.1.32"}},
                          {config(:sqlite), {:sqlite3, "~> 1.1"}},
                          {config(:stun), {:stun, "~> 1.0"}}], do:
       dep
@@ -281,6 +267,7 @@ defmodule Ejabberd.MixProject do
       ejabberd: [
         include_executables_for: [:unix],
         # applications: [runtime_tools: :permanent]
+        strip_beams: Mix.env() != :dev,
         steps: [&copy_extra_files/1, :assemble | maybe_tar]
       ]
     ]
