@@ -3,7 +3,7 @@
 %%% Purpose : Validator for ejabberd configuration options
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2025   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -546,13 +546,10 @@ sip_uri() ->
 -spec host() -> yconf:validator(binary()).
 host() ->
     fun(Domain) ->
-	    Host = ejabberd_config:get_myname(),
 	    Hosts = ejabberd_config:get_option(hosts),
-	    Domain1 = (binary())(Domain),
-	    Domain2 = misc:expand_keyword(<<"@HOST@">>, Domain1, Host),
-	    Domain3 = (domain())(Domain2),
+	    Domain3 = (domain())(Domain),
 	    case lists:member(Domain3, Hosts) of
-		true -> fail({route_conflict, Domain3});
+		true -> fail({route_conflict, Domain});
 		false -> Domain3
 	    end
     end.
