@@ -5,7 +5,7 @@
 %%% Created :  7 May 2006 by Mickael Remond <mremond@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2025   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -598,8 +598,9 @@ get_commands_spec() ->
 			args = [{node, atom}, {table, binary}, {page, integer}],
 			result = {res, any}},
 
-     #ejabberd_commands{name = mnesia_list_tables, tags = [internal, mnesia],
+     #ejabberd_commands{name = mnesia_list_tables, tags = [mnesia],
                         desc = "List of Mnesia tables",
+			note = "added in 25.03",
                         module = ?MODULE, function = mnesia_list_tables,
 			result = {tables, {list, {table, {tuple, [{name, atom},
                                                                 {storage_type, binary},
@@ -615,8 +616,10 @@ get_commands_spec() ->
                                                                 {value, binary}
                                                                ]}}}}},
 
-     #ejabberd_commands{name = mnesia_table_change_storage, tags = [internal, mnesia],
-                        desc = "Change storage type of a Mnesia table to: ram_copies, disc_copies, or disc_only_copies.",
+     #ejabberd_commands{name = mnesia_table_change_storage, tags = [mnesia],
+                        desc = "Change storage type of a Mnesia table",
+			note = "added in 25.03",
+			longdesc = "Storage type can be: `ram_copies`, `disc_copies`, `disc_only_copies`, `remote_copy`.",
                         module = ?MODULE, function = mnesia_table_change_storage,
 			args = [{table, binary}, {storage_type, binary}],
 			result = {res, restuple}},
@@ -663,7 +666,7 @@ status() ->
 	    {value, {_, _, Version}} ->
 		{ok, io_lib:format("ejabberd ~s is running in that node", [Version])}
 	end,
-    {Is_running, String1 ++ "  " ++String2}.
+    {Is_running, String1 ++ "\n" ++String2}.
 
 stop() ->
     _ = supervisor:terminate_child(ejabberd_sup, ejabberd_sm),
@@ -931,14 +934,14 @@ get_cluster_node_details(Node, RunningNodes) ->
     get_cluster_node_details2(Node, lists:member(Node, RunningNodes)).
 
 get_cluster_node_details2(Node, false) ->
-    {Node, "false", "", -1, -1, -1, "unknown"};
+    {Node, "false", "", -1, -1, -1, unknown};
 get_cluster_node_details2(Node, true) ->
     try ejabberd_cluster:call(Node, ejabberd_admin, get_cluster_node_details3, []) of
         Result -> Result
     catch
         E:R ->
             Status = io_lib:format("~p: ~p", [E, R]),
-            {Node, "true", Status, -1, -1, -1, "unknown"}
+            {Node, "true", Status, -1, -1, -1, unknown}
     end.
 
 get_cluster_node_details3() ->
@@ -1281,13 +1284,12 @@ is_my_host(Host) ->
 
 %% @format-begin
 
-%% mnesia:del_table_copy(Table, Node);
-%% mnesia:change_table_copy_type(Table, Node, Type);
-
 mnesia_table_change_storage(STable, SType) ->
     Table = binary_to_existing_atom(STable, latin1),
     Type =
         case SType of
+            <<"remote_copy">> ->
+                remote_copy;
             <<"ram_copies">> ->
                 ram_copies;
             <<"disc_copies">> ->
@@ -1297,7 +1299,24 @@ mnesia_table_change_storage(STable, SType) ->
             _ ->
                 false
         end,
-    mnesia:add_table_copy(Table, node(), Type).
+    Node = node(),
+    Result =
+        case Type of
+            false ->
+                "Nothing to do";
+            remote_copy ->
+                mnesia:del_table_copy(Table, Node),
+                "Deleted table copy";
+            _ ->
+                case mnesia:add_table_copy(Table, Node, Type) of
+                    {aborted, _} ->
+                        mnesia:change_table_copy_type(Table, Node, Type),
+                        "Changed table copy type";
+                    _ ->
+                        "Added table copy"
+                end
+        end,
+    {ok, Result}.
 
 mnesia_table_clear(STable) ->
     Table = binary_to_existing_atom(STable, latin1),
