@@ -5,7 +5,7 @@
 %%% Created : 12 Dec 2004 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2025   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -30,10 +30,10 @@
 
 -behaviour(ejabberd_auth).
 
--export([start/1, stop/1, set_password/3, try_register/3,
+-export([start/1, stop/1, set_password_multiple/3, try_register_multiple/3,
 	 get_users/2, count_users/2, get_password/2,
 	 remove_user/2, store_type/1, plain_password_required/1,
-	 export/1, which_users_exists/2]).
+	 export/1, which_users_exists/2, drop_password_type/2, set_password_instance/3]).
 -export([sql_schemas/0]).
 
 -include_lib("xmpp/include/scram.hrl").
@@ -49,7 +49,34 @@ start(Host) ->
     ok.
 
 sql_schemas() ->
-    [#sql_schema{
+    [
+	#sql_schema{
+	version = 2,
+	tables =
+	[#sql_table{
+	    name = <<"users">>,
+	    columns =
+	    [#sql_column{name = <<"username">>, type = text},
+		#sql_column{name = <<"server_host">>, type = text},
+		#sql_column{name = <<"type">>, type = smallint},
+		#sql_column{name = <<"password">>, type = text},
+		#sql_column{name = <<"serverkey">>, type = {text, 128},
+		    default = true},
+		#sql_column{name = <<"salt">>, type = {text, 128},
+		    default = true},
+		#sql_column{name = <<"iterationcount">>, type = integer,
+		    default = true},
+		#sql_column{name = <<"created_at">>, type = timestamp,
+		    default = true}],
+	    indices = [#sql_index{
+		columns = [<<"server_host">>, <<"username">>, <<"type">>],
+		unique = true}]}],
+	update = [
+	    {add_column, <<"users">>, <<"type">>},
+	    {update_primary_key,<<"users">>,
+		[<<"server_host">>, <<"username">>, <<"type">>]}
+	]},
+    #sql_schema{
         version = 1,
         tables =
             [#sql_table{
@@ -78,42 +105,87 @@ plain_password_required(Server) ->
 store_type(Server) ->
     ejabberd_auth:password_format(Server).
 
-set_password(User, Server, Password) ->
+hash_to_num(plain) -> 1;
+hash_to_num(sha) -> 2;
+hash_to_num(sha256) -> 3;
+hash_to_num(sha512) -> 4.
+
+num_to_hash(2) -> sha;
+num_to_hash(3) -> sha256;
+num_to_hash(4) -> sha512.
+
+set_password_instance(User, Server, #scram{hash = Hash, storedkey = SK, serverkey = SEK,
+					   salt = Salt, iterationcount = IC}) ->
+    F = fun() ->
+	set_password_scram_t(User, Server, Hash,
+			     SK, SEK, Salt, IC)
+	end,
+    case ejabberd_sql:sql_transaction(Server, F) of
+	{atomic, _} ->
+	    ok;
+	{aborted, _} ->
+	    {error, db_failure}
+    end;
+set_password_instance(User, Server, Plain) ->
+    F = fun() ->
+	set_password_t(User, Server, Plain)
+	end,
+    case ejabberd_sql:sql_transaction(Server, F) of
+	{atomic, _} ->
+	    ok;
+	{aborted, _} ->
+	    {error, db_failure}
+    end.
+
+set_password_multiple(User, Server, Passwords) ->
     F =
     fun() ->
-	case Password of
-	    #scram{hash = Hash, storedkey = SK, serverkey = SEK,
-		   salt = Salt, iterationcount = IC} ->
-		SK2 = scram_hash_encode(Hash, SK),
+	ejabberd_sql:sql_query_t(
+	    ?SQL("delete from users where username=%(User)s and %(Server)H")),
+	lists:foreach(
+	    fun(#scram{hash = Hash, storedkey = SK, serverkey = SEK,
+		       salt = Salt, iterationcount = IC}) ->
 		set_password_scram_t(
-		    User, Server,
-		    SK2, SEK, Salt, IC);
-	    _ ->
-		set_password_t(User, Server, Password)
-	end
+		    User, Server, Hash,
+		    SK, SEK, Salt, IC);
+	       (Plain) ->
+		   set_password_t(User, Server, Plain)
+	    end, Passwords)
     end,
     case ejabberd_sql:sql_transaction(Server, F) of
 	{atomic, _} ->
-	    {cache, {ok, Password}};
+	    {cache, {ok, Passwords}};
 	{aborted, _} ->
 	    {nocache, {error, db_failure}}
     end.
 
-try_register(User, Server, Password) ->
-    Res =
-    case Password of
-	#scram{hash = Hash, storedkey = SK, serverkey = SEK,
-	       salt = Salt, iterationcount = IC} ->
-	    SK2 = scram_hash_encode(Hash, SK),
-	    add_user_scram(
-		Server, User,
-		SK2, SEK, Salt, IC);
-	_ ->
-	    add_user(Server, User, Password)
-    end,
-    case Res of
-	{updated, 1} -> {cache, {ok, Password}};
-	_ -> {nocache, {error, exists}}
+try_register_multiple(User, Server, Passwords) ->
+    F =
+	fun() ->
+	    case ejabberd_sql:sql_query_t(
+		?SQL("select @(count(*))d from users where username=%(User)s and %(Server)H")) of
+		{selected, [{0}]} ->
+		    lists:foreach(
+			fun(#scram{hash = Hash, storedkey = SK, serverkey = SEK,
+				   salt = Salt, iterationcount = IC}) ->
+			    set_password_scram_t(
+				User, Server, Hash,
+				SK, SEK, Salt, IC);
+			   (Plain) ->
+			       set_password_t(User, Server, Plain)
+			end, Passwords),
+		    {cache, {ok, Passwords}};
+		{selected, _} ->
+		    {nocache, {error, exists}};
+		_ ->
+		    {nocache, {error, db_failure}}
+	    end
+	end,
+    case ejabberd_sql:sql_transaction(Server, F) of
+	{atomic, Res} ->
+	    Res;
+	{aborted, _} ->
+	    {nocache, {error, db_failure}}
     end.
 
 get_users(Server, Opts) ->
@@ -132,21 +204,41 @@ count_users(Server, Opts) ->
 
 get_password(User, Server) ->
     case get_password_scram(Server, User) of
-	{selected, [{Password, <<>>, <<>>, 0}]} ->
-	    {cache, {ok, Password}};
-	{selected, [{StoredKey, ServerKey, Salt, IterationCount}]} ->
-	    {Hash, SK} = case StoredKey of
-			     <<"sha256:", Rest/binary>> -> {sha256, Rest};
-			     <<"sha512:", Rest/binary>> -> {sha512, Rest};
-			     Other -> {sha, Other}
-			 end,
-	    {cache, {ok, #scram{storedkey = SK,
-				serverkey = ServerKey,
-				salt = Salt,
-				hash = Hash,
-				iterationcount = IterationCount}}};
 	{selected, []} ->
 	    {cache, error};
+	{selected, Passwords} ->
+	    Converted = lists:map(
+		fun({0, Password, <<>>, <<>>, 0}) ->
+		    update_password_type(User, Server, 1),
+		    Password;
+		   ({_, Password, <<>>, <<>>, 0}) ->
+		       Password;
+		   ({0, StoredKey, ServerKey, Salt, IterationCount}) ->
+		       {Hash, SK} = case StoredKey of
+					<<"sha256:", Rest/binary>> ->
+					    update_password_type(User, Server, 3, Rest),
+					    {sha256, Rest};
+					<<"sha512:", Rest/binary>> ->
+					    update_password_type(User, Server, 4, Rest),
+					    {sha512, Rest};
+					Other ->
+					    update_password_type(User, Server, 2),
+					    {sha, Other}
+				    end,
+		       #scram{storedkey = SK,
+			      serverkey = ServerKey,
+			      salt = Salt,
+			      hash = Hash,
+			      iterationcount = IterationCount};
+		   ({Type, StoredKey, ServerKey, Salt, IterationCount}) ->
+		       Hash = num_to_hash(Type),
+		       #scram{storedkey = StoredKey,
+			      serverkey = ServerKey,
+			      salt = Salt,
+			      hash = Hash,
+			      iterationcount = IterationCount}
+		end, Passwords),
+	    {cache, {ok, Converted}};
 	_ ->
 	    {nocache, error}
     end.
@@ -159,6 +251,13 @@ remove_user(User, Server) ->
 	    {error, db_failure}
     end.
 
+drop_password_type(LServer, Hash) ->
+    Type = hash_to_num(Hash),
+    ejabberd_sql:sql_query(
+	LServer,
+	?SQL("delete from users"
+	     " where type=%(Type)d and %(LServer)H")).
+
 scram_hash_encode(Hash, StoreKey) ->
     case Hash of
 	sha -> StoreKey;
@@ -166,12 +265,14 @@ scram_hash_encode(Hash, StoreKey) ->
 	sha512 -> <<"sha512:", StoreKey/binary>>
     end.
 
-set_password_scram_t(LUser, LServer,
+set_password_scram_t(LUser, LServer, Hash,
                      StoredKey, ServerKey, Salt, IterationCount) ->
+    Type = hash_to_num(Hash),
     ?SQL_UPSERT_T(
        "users",
        ["!username=%(LUser)s",
         "!server_host=%(LServer)s",
+	"!type=%(Type)d",
         "password=%(StoredKey)s",
         "serverkey=%(ServerKey)s",
         "salt=%(Salt)s",
@@ -182,40 +283,31 @@ set_password_t(LUser, LServer, Password) ->
        "users",
        ["!username=%(LUser)s",
         "!server_host=%(LServer)s",
+	"!type=1",
 	"password=%(Password)s",
 	"serverkey=''",
 	"salt=''",
 	"iterationcount=0"]).
 
+update_password_type(LUser, LServer, Type, Password) ->
+    ejabberd_sql:sql_query(
+	LServer,
+	?SQL("update users set type=%(Type)d, password=%(Password)s"
+	     " where username=%(LUser)s and type=0 and %(LServer)H")).
+
+update_password_type(LUser, LServer, Type) ->
+    ejabberd_sql:sql_query(
+	LServer,
+	?SQL("update users set type=%(Type)d"
+	     " where username=%(LUser)s and type=0 and %(LServer)H")).
+
 get_password_scram(LServer, LUser) ->
     ejabberd_sql:sql_query(
       LServer,
-      ?SQL("select @(password)s, @(serverkey)s, @(salt)s, @(iterationcount)d"
+      ?SQL("select @(type)d, @(password)s, @(serverkey)s, @(salt)s, @(iterationcount)d"
            " from users"
            " where username=%(LUser)s and %(LServer)H")).
 
-add_user_scram(LServer, LUser,
-               StoredKey, ServerKey, Salt, IterationCount) ->
-    ejabberd_sql:sql_query(
-      LServer,
-      ?SQL_INSERT(
-         "users",
-         ["username=%(LUser)s",
-          "server_host=%(LServer)s",
-          "password=%(StoredKey)s",
-          "serverkey=%(ServerKey)s",
-          "salt=%(Salt)s",
-          "iterationcount=%(IterationCount)d"])).
-
-add_user(LServer, LUser, Password) ->
-    ejabberd_sql:sql_query(
-      LServer,
-      ?SQL_INSERT(
-         "users",
-         ["username=%(LUser)s",
-          "server_host=%(LServer)s",
-          "password=%(Password)s"])).
-
 del_user(LServer, LUser) ->
     ejabberd_sql:sql_query(
       LServer,
@@ -224,7 +316,7 @@ del_user(LServer, LUser) ->
 list_users(LServer, []) ->
     ejabberd_sql:sql_query(
       LServer,
-      ?SQL("select @(username)s from users where %(LServer)H"));
+      ?SQL("select @(distinct username)s from users where %(LServer)H"));
 list_users(LServer, [{from, Start}, {to, End}])
     when is_integer(Start) and is_integer(End) ->
     list_users(LServer,
@@ -240,7 +332,7 @@ list_users(LServer, [{limit, Limit}, {offset, Offset}])
     when is_integer(Limit) and is_integer(Offset) ->
     ejabberd_sql:sql_query(
       LServer,
-      ?SQL("select @(username)s from users "
+      ?SQL("select @(distinct username)s from users "
            "where %(LServer)H "
            "order by username "
            "limit %(Limit)d offset %(Offset)d"));
@@ -252,7 +344,7 @@ list_users(LServer,
     SPrefix2 = <<SPrefix/binary, $%>>,
     ejabberd_sql:sql_query(
       LServer,
-      ?SQL("select @(username)s from users "
+      ?SQL("select @(distinct username)s from users "
            "where username like %(SPrefix2)s %ESCAPE and %(LServer)H "
            "order by username "
            "limit %(Limit)d offset %(Offset)d")).
@@ -269,11 +361,11 @@ users_number(LServer) ->
                              " where oid = 'users'::regclass::oid"));
                   _ ->
                       ejabberd_sql:sql_query_t(
-                        ?SQL("select @(count(*))d from users where %(LServer)H"))
+                        ?SQL("select @(count(distinct username))d from users where %(LServer)H"))
 	  end;
          (_Type, _) ->
               ejabberd_sql:sql_query_t(
-                ?SQL("select @(count(*))d from users where %(LServer)H"))
+                ?SQL("select @(count(distinct username))d from users where %(LServer)H"))
       end).
 
 users_number(LServer, [{prefix, Prefix}])
@@ -282,7 +374,7 @@ users_number(LServer, [{prefix, Prefix}])
     SPrefix2 = <<SPrefix/binary, $%>>,
     ejabberd_sql:sql_query(
       LServer,
-      ?SQL("select @(count(*))d from users "
+      ?SQL("select @(count(distinct username))d from users "
            "where username like %(SPrefix2)s %ESCAPE and %(LServer)H"));
 users_number(LServer, []) ->
     users_number(LServer).
@@ -290,7 +382,7 @@ users_number(LServer, []) ->
 which_users_exists(LServer, LUsers) when length(LUsers) =< 100 ->
     try ejabberd_sql:sql_query(
         LServer,
-        ?SQL("select @(username)s from users where username in %(LUsers)ls")) of
+        ?SQL("select @(distinct username)s from users where username in %(LUsers)ls")) of
         {selected, Matching} ->
             [U || {U} <- Matching];
         {error, _} = E ->
