@@ -1,5 +1,5 @@
 %%%----------------------------------------------------------------------
-%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2025   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -32,7 +32,7 @@ doc() ->
         desc =>
             ?T("List of one or more "
                "_`../configuration/basic.md#host-names|host names`_ "
-               "(or domains) that 'ejabberd' will serve. This is a "
+               "(or domains) that ejabberd will serve. This is a "
                "**mandatory** option.")}},
      {listen,
       #{value => "[Options, ...]",
@@ -222,7 +222,7 @@ doc() ->
                  "specified 'Pattern' according to the rules used by the "
                  "Unix shell.")}}]},
      {access_rules,
-      #{value => "{AccessName: {allow|deny: ACLRules|ACLName}}",
+      #{value => "{AccessName: {allow|deny: ACLName|ACLDefinition}}",
         desc =>
             ?T("This option defines "
                "_`basic.md#access-rules|Access Rules`_. "
@@ -307,7 +307,7 @@ doc() ->
       #{value => "true | false",
         desc =>
             ?T("Whether to allow installation of third-party modules or not. "
-               "See _`../../developer/extending-ejabberd/modules.md#ejabberd-contrib|ejabberd-contrib`_ "
+               "See _`../../admin/guide/modules.md#ejabberd-contrib|ejabberd-contrib`_ "
                "documentation section. "
                "The default value is 'true'.")}},
      {allow_multiple_connections,
@@ -368,7 +368,7 @@ doc() ->
         desc =>
             ?T("This is used by the contributed module "
 	       "'ejabberd_auth_http' that can be installed from the "
-	       "https://github.com/processone/ejabberd-contrib[ejabberd-contrib] "
+               "_`../../admin/guide/modules.md#ejabberd-contrib|ejabberd-contrib`_ "
                "Git repository. Please refer to that "
 	       "module's README file for details.")}},
      {auth_password_format,
@@ -399,6 +399,15 @@ doc() ->
            "You shouldn't change this if you already have passwords generated with "
            "a different algorithm - users that have such passwords will not be able "
            "to authenticate. The default value is 'sha'.")}},
+     {auth_stored_password_types,
+      #{value => "[plain | scram_sha1 | scram_sha256 | scram_sha512]",
+        note => "added in 25.03",
+        desc =>
+        ?T("List of password types that should be stored simultaneously for each user in database. "
+           "When the user sets the account password, database will be updated to store the password in formats "
+           "compatible with each type listed here. This can be used to migrate user passwords "
+           "to a more secure format. If this option if set, it will override values set in _`auth_scram_hash`_ "
+           "and _`auth_password_format`_ options. The default value is `[]`.")}},
      {auth_external_user_exists_check,
       #{value => "true | false",
         note => "added in 23.10",
@@ -472,8 +481,8 @@ doc() ->
         note => "improved in 23.01",
         desc =>
             ?T("Full path to a script that generates _`basic.md#captcha|CAPTCHA`_ images. "
-               "'@VERSION@' is replaced with ejabberd version number in 'XX.YY' format. "
-               "'@SEMVER@' is replaced with ejabberd version number in semver format "
+               "The keyword '@VERSION@' is replaced with ejabberd version number in 'XX.YY' format. "
+               "The keyword '@SEMVER@' is replaced with ejabberd version number in semver format "
                "when compiled with Elixir's mix, or XX.YY format otherwise. "
                "Alternatively, it can be the name of a module that implements ejabberd CAPTCHA support. "
                "There is no default value: when this option is not "
@@ -534,18 +543,28 @@ doc() ->
             ?T("A list of Erlang nodes to connect on ejabberd startup. "
                "This option is mostly intended for ejabberd customization "
                "and sophisticated setups. The default value is an empty list.")}},
-     {define_macro,
-      #{value => "{MacroName: MacroValue}",
+     {define_keyword,
+      #{value => "{NAME: Value}",
+        note => "added in 25.03",
         desc =>
-            ?T("Defines a "
-               "_`../configuration/file-format.md#macros-in-configuration-file|macro`_. "
-               "The value can be any valid arbitrary "
-               "YAML value. For convenience, it's recommended to define "
-               "a 'MacroName' in capital letters. Duplicated macros are not allowed. "
-               "Macros are processed after additional configuration files have "
-               "been included, so it is possible to use macros that are defined "
-               "in configuration files included before the usage. "
-               "It is possible to use a 'MacroValue' in the definition of another macro."),
+            ?T("Allows to define configuration "
+                "_`../configuration/file-format.md#macros-and-keywords|keywords`_. "),
+        example =>
+            ["define_keyword:",
+             "  SQL_USERNAME: \"eja.global\"",
+             "",
+             "host_config:",
+             "  localhost:",
+             "    define_keyword:",
+             "      SQL_USERNAME: \"eja.localhost\"",
+             "",
+             "sql_username: \"prefix.@SQL_USERNAME@\""]}},
+     {define_macro,
+      #{value => "{NAME: Value}",
+        note => "improved in 25.03",
+        desc =>
+            ?T("Allows to define configuration "
+                "_`../configuration/file-format.md#macros-and-keywords|macros`_. "),
         example =>
             ["define_macro:",
              "  DEBUG: debug",
@@ -719,7 +738,7 @@ doc() ->
         note => "added in 23.10",
         desc =>
             ?T("Modules to install from "
-               "_`../../developer/extending-ejabberd/modules.md#ejabberd-contrib|ejabberd-contrib`_ "
+               "_`../../admin/guide/modules.md#ejabberd-contrib|ejabberd-contrib`_ "
                "at start time. "
                "The default value is an empty list of modules: '[]'.")}},
      {jwt_auth_only_rule,
@@ -845,18 +864,18 @@ doc() ->
                "as alternatives for getting the JID, where 'Attr' is "
                "an LDAP attribute which holds the user's part of the JID and "
                "'AttrFormat' must contain one and only one pattern variable "
-               "\"%u\" which will be replaced by the user's part of the JID. "
-               "For example, \"%u@example.org\". If the value is in the form "
-               "of '[Attr]' then 'AttrFormat' is assumed to be \"%u\".")}},
+               "'\"%u\"' which will be replaced by the user's part of the JID. "
+               "For example, '\"%u@example.org\"'. If the value is in the form "
+               "of '[Attr]' then 'AttrFormat' is assumed to be '\"%u\"'.")}},
      {ldap_filter,
       #{value => ?T("Filter"),
         desc =>
             ?T("An LDAP filter as defined in "
                "https://tools.ietf.org/html/rfc4515[RFC4515]. "
                "There is no default value. Example: "
-               "\"(&(objectClass=shadowAccount)(memberOf=XMPP Users))\". "
+               "'\"(&(objectClass=shadowAccount)(memberOf=XMPP Users))\"'. "
                "NOTE: don't forget to close brackets and don't use superfluous "
-               "whitespaces. Also you must not use \"uid\" attribute in the "
+               "whitespaces. Also you must not use '\"uid\"' attribute in the "
                "filter because this attribute will be appended to the filter "
                "automatically.")}},
      {ldap_dn_filter,
@@ -866,11 +885,11 @@ doc() ->
                "filter. The filter performs an additional LDAP lookup to make "
                "the complete result. This is useful when you are unable to "
                "define all filter rules in 'ldap_filter'. You can define "
-               "\"%u\", \"%d\", \"%s\" and \"%D\" pattern variables in 'Filter': "
-               "\"%u\" is replaced by a user's part of the JID, \"%d\" is "
-               "replaced by the corresponding domain (virtual host), all \"%s\" "
+               "'\"%u\"', '\"%d\"', '\"%s\"' and '\"%D\"' pattern variables in 'Filter: "
+               "\"%u\"' is replaced by a user's part of the JID, '\"%d\"' is "
+               "replaced by the corresponding domain (virtual host), all '\"%s\"' "
                "variables are consecutively replaced by values from the attributes "
-               "in 'FilterAttrs' and \"%D\" is replaced by Distinguished Name from "
+               "in 'FilterAttrs' and '\"%D\"' is replaced by Distinguished Name from "
                "the result set. There is no default value, which means the "
                "result is not filtered. WARNING: Since this filter makes "
                "additional LDAP lookups, use it only as the last resort: "
@@ -1322,7 +1341,7 @@ doc() ->
              "  normal: 1000",
              "  fast: 50000"]}},
      {shaper_rules,
-      #{value => "{ShaperRuleName: {Number|ShaperName: ACLRule|ACLName}}",
+      #{value => "{ShaperRuleName: {Number|ShaperName: ACLName|ACLDefinition}}",
         desc =>
             ?T("This option defines "
                "_`../configuration/basic.md#shaper-rules|shaper rules`_ "
