@@ -5,7 +5,7 @@
 %%% Created : 15 Aug 2023 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2025   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -410,18 +410,21 @@ sqlite_table_copy_t(SchemaInfo, Table) ->
     NewTableName = <<"new_", TableName/binary>>,
     NewTable = Table#sql_table{name = NewTableName},
     create_table_t(SchemaInfo, NewTable),
-    SQL2 = <<"INSERT INTO ", NewTableName/binary,
-             " SELECT * FROM ", TableName/binary>>,
+    Columns = lists:join(<<",">>,
+                         lists:map(fun(C) -> escape_name(SchemaInfo, C#sql_column.name) end,
+                                   Table#sql_table.columns)),
+    SQL2 = [<<"INSERT INTO ">>, NewTableName,
+            <<" SELECT ">>, Columns, <<" FROM ">>, TableName],
     ?INFO_MSG("Copying table ~s to ~s:~n~s~n",
               [TableName, NewTableName, SQL2]),
     ejabberd_sql:sql_query_t(SQL2),
     SQL3 = <<"DROP TABLE ", TableName/binary>>,
     ?INFO_MSG("Droping old table ~s:~n~s~n",
-              [TableName, SQL2]),
+              [TableName, SQL3]),
     ejabberd_sql:sql_query_t(SQL3),
     SQL4 = <<"ALTER TABLE ", NewTableName/binary,
              " RENAME TO ", TableName/binary>>,
-    ?INFO_MSG("Renameing table ~s to ~s:~n~s~n",
+    ?INFO_MSG("Renaming table ~s to ~s:~n~s~n",
               [NewTableName, TableName, SQL4]),
     ejabberd_sql:sql_query_t(SQL4).
 
