@@ -5,7 +5,7 @@
 %%% Created :  9 Apr 2004 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2025   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -369,7 +369,7 @@ make_xhtml(Els, Host, Node, Username, #request{lang = Lang} = R, JID, Level) ->
 				 [?XE(<<"p">>,
 				  [?AC(<<"https://www.ejabberd.im/">>, <<"ejabberd">>),
 				   ?C(<<" ">>), ?C(ejabberd_option:version()),
-				   ?C(<<" (c) 2002-2024 ">>),
+				   ?C(<<" (c) 2002-2025 ">>),
 				   ?AC(<<"https://www.process-one.net/">>, <<"ProcessOne, leader in messaging and push solutions">>)]
                                  )])])])]}}.
 
@@ -584,12 +584,17 @@ process_admin(Host, #request{path = [<<"users">> | RPath], lang = Lang} = R, AJI
 process_admin(Host, #request{path = [<<"online-users">> | RPath], lang = Lang} = R, AJID)
     when is_binary(Host) ->
     Level = 3 + length(RPath),
-    Res = [make_command(connected_users_vhost,
+    Set = [make_command(kick_users,
+                        R,
+                        [{<<"host">>, Host}],
+                        [{style, danger}, {force_execution, false}])],
+    timer:sleep(200), % small delay after kicking users before getting the updated list
+    Get = [make_command(connected_users_vhost,
                         R,
                         [{<<"host">>, Host}],
                         [{table_options, {100, RPath}},
                          {result_links, [{sessions, user, Level, <<"">>}]}])],
-    make_xhtml([?XCT(<<"h1">>, ?T("Online Users"))] ++ Res, Host, R, AJID, Level);
+    make_xhtml([?XCT(<<"h1">>, ?T("Online Users"))] ++ Set ++ Get, Host, R, AJID, Level);
 process_admin(Host,
               #request{path = [<<"last-activity">>],
                        q = Query,
@@ -2359,11 +2364,21 @@ format_result([], {_Name, {list, _ElementsDef}}) ->
     "";
 format_result([FirstElement | Elements], {_Name, {list, ElementsDef}}) ->
     Separator = ",",
-    [format_result(FirstElement, ElementsDef) | lists:map(fun(Element) ->
-                                                             [Separator | format_result(Element,
-                                                                                        ElementsDef)]
-                                                          end,
-                                                          Elements)];
+    Head = format_result(FirstElement, ElementsDef),
+    Tail =
+        lists:map(fun(Element) -> [Separator | format_result(Element, ElementsDef)] end,
+                  Elements),
+    [Head | Tail];
+format_result([], {_Name, {tuple, _ElementsDef}}) ->
+    "";
+format_result(Value, {_Name, {tuple, [FirstDef | ElementsDef]}}) ->
+    [FirstElement | Elements] = tuple_to_list(Value),
+    Separator = ":",
+    Head = format_result(FirstElement, FirstDef),
+    Tail =
+        lists:map(fun(Element) -> [Separator | format_result(Element, ElementsDef)] end,
+                  Elements),
+    [Head | Tail];
 format_result(Value, _ResultFormat) when is_atom(Value) ->
     misc:atom_to_binary(Value);
 format_result(Value, _ResultFormat) when is_list(Value) ->
