@@ -5,7 +5,7 @@
 %%% Created : 19 Feb 2015 by Christophe Romain <christophe.romain@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2006-2024   ProcessOne
+%%% ejabberd, Copyright (C) 2006-2025   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -573,7 +573,7 @@ compile_and_install(Module, Spec, Config) ->
         true ->
             case compile_deps(SrcDir) of
                 ok ->
-                    case compile(SrcDir) of
+                    case compile(SrcDir, filename:join(SrcDir, "deps")) of
                         ok -> install(Module, Spec, SrcDir, LibDir, Config);
                         Error -> Error
                     end;
@@ -589,25 +589,31 @@ compile_and_install(Module, Spec, Config) ->
     end.
 
 compile_deps(LibDir) ->
-    Deps = filename:join(LibDir, "deps"),
-    case filelib:is_dir(Deps) of
+    DepsDir = filename:join(LibDir, "deps"),
+    case filelib:is_dir(DepsDir) of
         true -> ok;  % assume deps are included
         false -> fetch_rebar_deps(LibDir)
     end,
-    Rs = [compile(Dep) || Dep <- filelib:wildcard(filename:join(Deps, "*"))],
+    Rs = [compile(Dep, DepsDir) || Dep <- filelib:wildcard(filename:join(DepsDir, "*"))],
     compile_result(Rs).
 
-compile(LibDir) ->
+compile(LibDir, DepsDir) ->
     Bin = filename:join(LibDir, "ebin"),
     Lib = filename:join(LibDir, "lib"),
     Src = filename:join(LibDir, "src"),
-    Includes = [{i, Inc} || Inc <- filelib:wildcard(LibDir++"/../../**/include")],
-    Options = [{outdir, Bin}, {i, LibDir++"/.."} | Includes ++ compile_options()],
+    Includes = [ {i, Inc} || Inc <- filelib:wildcard(DepsDir++"/**/include") ],
+    Options = [ {outdir, Bin},
+                {i, LibDir++"/.."},
+                {i, filename:join(LibDir, "include")}
+              | Includes ++ compile_options()],
+    ?DEBUG("compile options: ~p", [Options]),
     filelib:ensure_dir(filename:join(Bin, ".")),
     [copy(App, filename:join(Bin, filename:basename(App, ".src"))) || App <- filelib:wildcard(Src++"/*.app*")],
     compile_c_files(LibDir),
+    ErlFiles = filelib:wildcard(Src++"/**/*.erl"),
+    ?DEBUG("erl files to compile: ~p", [ErlFiles]),
     Er = [compile_erlang_file(Bin, File, Options)
-          || File <- filelib:wildcard(Src++"/**/*.erl")],
+          || File <- ErlFiles],
     Ex = compile_elixir_files(Bin, filelib:wildcard(Lib ++ "/**/*.ex")),
     compile_result(lists:flatten([Er, Ex])).
 
@@ -792,7 +798,7 @@ rebar_dep({App, Version, Git}) when Version /= ".*" ->
     Help = os:cmd("mix hex.package"),
     case string:find(Help, "mix hex.package fetch") /= nomatch of
         true ->
-            {App, "mix hex.package fetch "++AppS++" "++Version++" --unpack"};
+            {App, "mix hex.package fetch "++AppS++" "++Version++" --unpack --output "++AppS};
         false ->
             io:format("I'll download ~p using git because I can't use Mix "
                       "to fetch from hex.pm:~n~s", [AppS, Help]),
