@@ -5,7 +5,7 @@
 %%% Created : 24 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2025   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -45,6 +45,7 @@
 -include("logger.hrl").
 -include_lib("stdlib/include/ms_transform.hrl").
 -include("ejabberd_stacktrace.hrl").
+-include("ejabberd_commands.hrl").
 
 -record(ejabberd_module,
         {module_host = {undefined, <<"">>} :: {atom(), binary()},
@@ -63,6 +64,11 @@
         {hook, atom(), atom(), integer()} |
         {hook, atom(), atom(), integer(), binary() | global} |
         {hook, atom(), module(), atom(), integer()} |
+        {hook_subscribe, atom(), atom(), [any()]} |
+        {hook_subscribe, atom(), atom(), [any()], binary() | global} |
+        {hook_subscribe, atom(), module(), atom(), [any()]} |
+        {hook_subscribe, atom(), module(), atom(), [any()], binary() | global} |
+        {commands, [ejabberd_commands()]} |
         {iq_handler, component(), binary(), atom()} |
         {iq_handler, component(), binary(), module(), atom()}.
 -export_type([registration/0]).
@@ -346,6 +352,16 @@ add_registrations(Host, Module, Registrations) ->
               ejabberd_hooks:add(Hook, Host1, Module, Function, Seq);
          ({hook, Hook, Module1, Function, Seq}) when is_integer(Seq) ->
               ejabberd_hooks:add(Hook, Host, Module1, Function, Seq);
+         ({hook_subscribe, Hook, Function, InitArg}) ->
+              ejabberd_hooks:subscribe(Hook, Host, Module, Function, InitArg);
+         ({hook_subscribe, Hook, Function, InitArg, Host1}) when is_binary(Host1) or (Host1 == global) ->
+              ejabberd_hooks:subscribe(Hook, Host1, Module, Function, InitArg);
+         ({hook_subscribe, Hook, Module1, Function, InitArg}) ->
+              ejabberd_hooks:subscribe(Hook, Host, Module1, Function, InitArg);
+         ({hook_subscribe, Hook, Module1, Function, InitArg, Host1}) ->
+              ejabberd_hooks:subscribe(Hook, Host1, Module1, Function, InitArg);
+         ({commands, Commands}) ->
+              ejabberd_commands:register_commands(Host, Module, Commands);
          ({iq_handler, Component, NS, Function}) ->
               gen_iq_handler:add_iq_handler(
                 Component, Host, NS, Module, Function);
@@ -363,6 +379,16 @@ del_registrations(Host, Module, Registrations) ->
               ejabberd_hooks:delete(Hook, Host1, Module, Function, Seq);
          ({hook, Hook, Module1, Function, Seq}) when is_integer(Seq) ->
               ejabberd_hooks:delete(Hook, Host, Module1, Function, Seq);
+         ({hook_subscribe, Hook, Function, InitArg}) ->
+              ejabberd_hooks:unsubscribe(Hook, Host, Module, Function, InitArg);
+         ({hook_subscribe, Hook, Function, InitArg, Host1}) when is_binary(Host1) or (Host1 == global) ->
+              ejabberd_hooks:unsubscribe(Hook, Host1, Module, Function, InitArg);
+         ({hook_subscribe, Hook, Module1, Function, InitArg}) ->
+              ejabberd_hooks:unsubscribe(Hook, Host, Module1, Function, InitArg);
+         ({hook_subscribe, Hook, Module1, Function, InitArg, Host1}) ->
+              ejabberd_hooks:unsubscribe(Hook, Host1, Module1, Function, InitArg);
+         ({commands, Commands}) ->
+              ejabberd_commands:unregister_commands(Host, Module, Commands);
          ({iq_handler, Component, NS, _Function}) ->
               gen_iq_handler:remove_iq_handler(Component, Host, NS);
          ({iq_handler, Component, NS, _Module, _Function}) ->
@@ -551,10 +577,10 @@ validator(Host, Module, Opts) ->
 		  lists:mapfoldl(
 		    fun({Opt, Def}, {DAcc1, VAcc1}) ->
 			    {[], {DAcc1#{Opt => Def},
-				  VAcc1#{Opt => get_opt_type(Module, M, Opt)}}};
+				  VAcc1#{Opt => get_opt_type(Host, Module, M, Opt)}}};
 		       (Opt, {DAcc1, VAcc1}) ->
 			    {[Opt], {DAcc1,
-				     VAcc1#{Opt => get_opt_type(Module, M, Opt)}}}
+				     VAcc1#{Opt => get_opt_type(Host, Module, M, Opt)}}}
 		    end, {DAcc, VAcc}, DefOpts)
 	  end, {#{}, #{}}, get_defaults(Host, Module, Opts)),
     econf:and_then(
@@ -604,11 +630,16 @@ get_defaults(Host, Module, Opts) ->
 	       false
        end, DefaultOpts)].
 
--spec get_opt_type(module(), module(), atom()) -> econf:validator().
-get_opt_type(Mod, SubMod, Opt) ->
-    try SubMod:mod_opt_type(Opt)
+-spec get_opt_type(binary(), module(), module(), atom()) -> econf:validator().
+get_opt_type(Host, Mod, SubMod, Opt) ->
+    Type = try SubMod:mod_opt_type(Opt)
     catch _:_ -> Mod:mod_opt_type(Opt)
-    end.
+    end,
+    econf:and_then(
+      fun(B) ->
+              ejabberd_config:replace_keywords(Host, B)
+      end,
+      Type).
 
 -spec sort_modules(binary(), [{module(), opts()}]) -> {ok, [{module(), opts(), integer()}]}.
 sort_modules(Host, ModOpts) ->
