@@ -8,7 +8,7 @@
 %%% Created : 30 Mar 2017 by Evgeny Khramtsov <ekhramtsov@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2025   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -141,7 +141,7 @@ json_decode(Bin) ->
 -else.
 json_encode_with_kv_lists(Term) ->
     iolist_to_binary(json:encode(Term,
-		     fun([{_, _} | _] = Val, Encoder) ->
+		     fun({Val}, Encoder) when is_list(Val) ->
 			 json:encode_key_value_list(Val, Encoder);
 			(Val, Encoder) ->
 			    json:encode_value(Val, Encoder)
