@@ -5,7 +5,7 @@
 %%% Created :  9 Aug 2017 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2025   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -48,10 +48,10 @@
 %%%
 
 start(_Host, _Opts) ->
-    ejabberd_commands:register_commands(?MODULE, get_commands_spec()).
+    {ok, [{commands, get_commands_spec()}]}.
 
 stop(_Host) ->
-    ejabberd_commands:unregister_commands(get_commands_spec()).
+    ok.
 
 reload(_Host, _NewOpts, _OldOpts) ->
     ok.
@@ -286,6 +286,7 @@ update_tables(State) ->
         true ->
             drop_index(State, "private_storage", "i_private_storage_username"),
             drop_index(State, "private_storage", "i_private_storage_username_namespace"),
+            drop_pkey(State, "private_storage"),
             add_pkey(State, "private_storage", ["server_host", "username", "namespace"]),
             drop_sh_default(State, "private_storage");
         false ->
@@ -342,6 +343,7 @@ update_tables(State) ->
         true ->
             drop_index(State, "sm", "i_sm_sid"),
             drop_index(State, "sm", "i_sm_username"),
+            drop_pkey(State, "sm"),
             add_pkey(State, "sm", ["usec", "pid"]),
             create_index(State, "sm", "i_sm_sh_username", ["server_host", "username"]),
             drop_sh_default(State, "sm");
@@ -592,8 +594,22 @@ mssql_clustered(_) -> "CLUSTERED ".
 mysql_keylen(_, "bare_peer") -> "(191)";
 mysql_keylen(_, "channel") -> "(191)";
 mysql_keylen(_, "domain") -> "(75)";
+mysql_keylen(_, "grp") -> "(191)"; %% in mysql*.sql this is text, not varchar(191)
 mysql_keylen(_, "jid") -> "(75)";
+mysql_keylen(_, "lbday") -> "(191)";
+mysql_keylen(_, "lctry") -> "(191)";
+mysql_keylen(_, "lemail") -> "(191)";
+mysql_keylen(_, "lfamily") -> "(191)";
+mysql_keylen(_, "lfn") -> "(191)";
+mysql_keylen(_, "lgiven") -> "(191)";
+mysql_keylen(_, "llocality") -> "(191)";
+mysql_keylen(_, "lmiddle") -> "(191)";
+mysql_keylen(_, "lnickname") -> "(191)";
+mysql_keylen(_, "lorgname") -> "(191)";
+mysql_keylen(_, "lorgunit") -> "(191)";
+mysql_keylen(_, "lusername") -> "(191)";
 mysql_keylen(_, "name") -> "(75)";
+mysql_keylen(_, "namespace") -> "(191)";
 mysql_keylen(_, "node") -> "(75)";
 mysql_keylen(_, "peer") -> "(191)";
 mysql_keylen(_, "pid") -> "(75)";
