@@ -168,7 +168,7 @@ find_handler_port_path(Tls, Module) ->
       fun({{Port, _, _},
            ejabberd_http,
            #{tls := ThisTls, request_handlers := Handlers}})
-            when (Tls == any) or (Tls == ThisTls) ->
+            when is_integer(Port) and ((Tls == any) or (Tls == ThisTls)) ->
               case lists:keyfind(Module, 2, Handlers) of
                   false -> false;
                   {Path, Module} -> {true, {ThisTls, Port, Path}}
