@@ -5,7 +5,7 @@
 %%% Created :  4 Jul 2013 by Evgeniy Khramtsov <ekhramtsov@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2013-2024   ProcessOne
+%%% ejabberd, Copyright (C) 2013-2025   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -28,6 +28,7 @@
 -protocol({xep, 313, '0.6.1', '15.06', "complete", ""}).
 -protocol({xep, 334, '0.2', '16.01', "complete", ""}).
 -protocol({xep, 359, '0.5.0', '15.09', "complete", ""}).
+-protocol({xep, 424, '0.4.2', '24.02', "partial", "Tombstones not implemented"}).
 -protocol({xep, 425, '0.3.0', '24.06', "complete", ""}).
 -protocol({xep, 441, '0.2.0', '15.06', "complete", ""}).
 
@@ -182,7 +183,7 @@ start(Host, Opts) ->
 		    ejabberd_hooks:add(check_create_room, Host, ?MODULE,
 				       check_create_room, 50)
 	    end,
-	    ejabberd_commands:register_commands(?MODULE, get_commands_spec()),
+	    ejabberd_commands:register_commands(Host, ?MODULE, get_commands_spec()),
 	    ok;
 	Err ->
 	    Err
@@ -263,12 +264,7 @@ stop(Host) ->
 	    ejabberd_hooks:delete(check_create_room, Host, ?MODULE,
 				  check_create_room, 50)
     end,
-    case gen_mod:is_loaded_elsewhere(Host, ?MODULE) of
-        false ->
-            ejabberd_commands:unregister_commands(get_commands_spec());
-        true ->
-            ok
-    end.
+    ejabberd_commands:unregister_commands(Host, ?MODULE, get_commands_spec()).
 
 reload(Host, NewOpts, OldOpts) ->
     NewMod = gen_mod:db_mod(NewOpts, ?MODULE),
@@ -1760,13 +1756,17 @@ mod_options(Host) ->
 
 mod_doc() ->
     #{desc =>
-          ?T("This module implements "
+          [?T("This module implements "
              "https://xmpp.org/extensions/xep-0313.html"
              "[XEP-0313: Message Archive Management] and "
              "https://xmpp.org/extensions/xep-0441.html"
              "[XEP-0441: Message Archive Management Preferences]. "
              "Compatible XMPP clients can use it to store their "
-             "chat history on the server."),
+             "chat history on the server."), "",
+           ?T("NOTE: Mnesia backend for mod_mam is not recommended: it's limited "
+             "to 2GB and often gets corrupted when reaching this limit. "
+             "SQL backend is recommended. Namely, for small servers SQLite "
+             "is a preferred choice because it's very easy to configure.")],
       opts =>
           [{access_preferences,
             #{value => ?T("AccessName"),
