@@ -5,7 +5,7 @@
 %%% Created : 19 Mar 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2025   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -25,6 +25,7 @@
 -module(mod_muc).
 -author('alexey@process-one.net').
 -protocol({xep, 45, '1.25', '0.5.0', "complete", ""}).
+-protocol({xep, 249, '1.2', '0.5.0', "complete", ""}).
 -ifndef(GEN_SERVER).
 -define(GEN_SERVER, gen_server).
 -endif.
@@ -1765,8 +1766,10 @@ mod_doc() ->
                        "The default value is an empty string.")}},
              {enable_hats,
               #{value => "true | false",
+                note => "improved in 25.03",
                 desc =>
                     ?T("Allow extended roles as defined in XEP-0317 Hats. "
+                       "Check the _`../../tutorials/muc-hats.md|MUC Hats`_ tutorial. "
                        "The default value is 'false'.")}},
              {lang,
               #{value => ?T("Language"),
@@ -1833,6 +1836,11 @@ mod_doc() ->
                 desc =>
                     ?T("A custom vCard for the room. See the equivalent mod_muc option."
                        "The default value is an empty string.")}},
+             {vcard_xupdate,
+              #{value => "undefined | external | AvatarHash",
+                desc =>
+                    ?T("Set the hash of the avatar image. "
+                       "The default value is 'undefined'.")}},
              {voice_request_min_interval,
               #{value => ?T("Number"),
                 desc =>
