@@ -1,11 +1,11 @@
 %%%----------------------------------------------------------------------
 %%% File    : mod_muc_admin.erl
-%%% Author  : Badlop <badlop@ono.com>
+%%% Author  : Badlop <badlop@process-one.net>
 %%% Purpose : Tools for additional MUC administration
-%%% Created : 8 Sep 2007 by Badlop <badlop@ono.com>
+%%% Created : 8 Sep 2007 by Badlop <badlop@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2025   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -24,7 +24,7 @@
 %%%----------------------------------------------------------------------
 
 -module(mod_muc_admin).
--author('badlop@ono.com').
+-author('badlop@process-one.net').
 
 -behaviour(gen_mod).
 
@@ -69,8 +69,8 @@
 %%----------------------------
 
 start(_Host, _Opts) ->
-    ejabberd_commands:register_commands(?MODULE, get_commands_spec()),
-    {ok, [{hook, webadmin_menu_main, web_menu_main, 50, global},
+    {ok, [{commands, get_commands_spec()},
+          {hook, webadmin_menu_main, web_menu_main, 50, global},
 	  {hook, webadmin_page_main, web_page_main, 50, global},
 	  {hook, webadmin_menu_host, web_menu_host, 50},
 	  {hook, webadmin_page_host, web_page_host, 50},
@@ -78,13 +78,8 @@ start(_Host, _Opts) ->
 	  {hook, webadmin_page_hostuser, web_page_hostuser, 50}
          ]}.
 
-stop(Host) ->
-    case gen_mod:is_loaded_elsewhere(Host, ?MODULE) of
-        false ->
-            ejabberd_commands:unregister_commands(get_commands_spec());
-        true ->
-            ok
-    end.
+stop(_Host) ->
+    ok.
 
 reload(_Host, _NewOpts, _OldOpts) ->
     ok.
@@ -194,14 +189,20 @@ get_commands_spec() ->
      #ejabberd_commands{name = create_room_with_opts, tags = [muc_room, muc_sub],
 		       desc = "Create a MUC room name@service in host with given options",
 		       longdesc =
-                        "The syntax of `affiliations` is: `Type:JID,Type:JID`. "
-                        "The syntax of `subscribers` is: `JID:Nick:Node:Node2:Node3,JID:Nick:Node`.",
+                        "Options `affiliations` and `subscribers` are lists of tuples. "
+                        "The tuples in the list are separated with `;` and "
+                        "the elements in each tuple are separated with `=` "
+                        "(until ejabberd 24.12 the separators were `,` and `:` respectively). "
+                        "Each subscriber can have one or more nodes. "
+                        "In summary, `affiliations` is like `Type1=JID1;Type2=JID2` "
+                        "and `subscribers` is like `JID1=Nick1=Node1A=Node1B=Node1C;JID2=Nick2=Node2`.",
+                       note = "modified in 25.03",
 		       module = ?MODULE, function = create_room_with_opts,
 		       args_desc = ["Room name", "MUC service", "Server host", "List of options"],
 		       args_example = ["room1", "conference.example.com", "localhost",
 				       [{"members_only","true"},
-                                        {"affiliations", "owner:bob@example.com,member:peter@example.com"},
-                                        {"subscribers", "bob@example.com:Bob:messages:subject,anne@example.com:Anne:messages"}]],
+                                        {"affiliations", "owner=user1@localhost;member=user2@localhost"},
+                                        {"subscribers", "user3@localhost=User3=messages=subject;user4@localhost=User4=messages"}]],
 		       args = [{room, binary}, {service, binary},
 			       {host, binary},
 			       {options, {list,
@@ -1722,9 +1723,8 @@ format_room_option(OptionString, ValueString) ->
 		password -> ValueString;
 		subject ->ValueString;
 		subject_author ->ValueString;
-		presence_broadcast ->misc:expr_to_term(ValueString);
-		max_users -> binary_to_integer(ValueString);
-		voice_request_min_interval -> binary_to_integer(ValueString);
+		max_users -> try_convert_integer(Option, ValueString);
+		voice_request_min_interval -> try_convert_integer(Option, ValueString);
 		vcard -> ValueString;
 		vcard_xupdate when ValueString /= <<"undefined">>,
 				   ValueString /= <<"external">> ->
@@ -1732,19 +1732,50 @@ format_room_option(OptionString, ValueString) ->
 		lang -> ValueString;
 		pubsub -> ValueString;
 		affiliations ->
-		    [parse_affiliation_string(Opt) || Opt <- str:tokens(ValueString, <<",">>)];
+		    [parse_affiliation_string(Opt) || Opt <- str:tokens(ValueString, <<";,">>)];
 		subscribers ->
-		    [parse_subscription_string(Opt) || Opt <- str:tokens(ValueString, <<",">>)];
-		_ -> misc:binary_to_atom(ValueString)
+		    [parse_subscription_string(Opt) || Opt <- str:tokens(ValueString, <<";,">>)];
+                allow_private_messages_from_visitors when
+                      (ValueString == <<"anyone">>) or
+                      (ValueString == <<"moderators">>) or
+                      (ValueString == <<"nobody">>) -> binary_to_existing_atom(ValueString, utf8);
+                allowpm when
+                      (ValueString == <<"anyone">>) or
+                      (ValueString == <<"participants">>) or
+                      (ValueString == <<"moderators">>) or
+                      (ValueString == <<"none">>) -> binary_to_existing_atom(ValueString, utf8);
+		presence_broadcast when
+                      (ValueString == <<"participant">>) or
+                      (ValueString == <<"moderator">>) or
+                      (ValueString == <<"visitor">>) -> binary_to_existing_atom(ValueString, utf8);
+		_ when ValueString == <<"true">> -> true;
+		_ when ValueString == <<"false">> -> false;
+                _ -> throw_error(Option, ValueString)
 	    end,
     {Option, Value}.
 
+try_convert_integer(Option, ValueString) ->
+    try binary_to_integer(ValueString) of
+	I when is_integer(I) -> I
+    catch _:badarg ->
+        throw_error(Option, ValueString)
+    end.
+
+throw_error(O, V) ->
+    throw({error, "Invalid value for that option", O, V}).
+
 parse_affiliation_string(String) ->
     {Type, JidS} = case String of
+                       %% Old syntax
 		       <<"owner:", Jid/binary>> -> {owner, Jid};
 		       <<"admin:", Jid/binary>> -> {admin, Jid};
 		       <<"member:", Jid/binary>> -> {member, Jid};
 		       <<"outcast:", Jid/binary>> -> {outcast, Jid};
+                       %% New syntax
+		       <<"owner=", Jid/binary>> -> {owner, Jid};
+		       <<"admin=", Jid/binary>> -> {admin, Jid};
+		       <<"member=", Jid/binary>> -> {member, Jid};
+		       <<"outcast=", Jid/binary>> -> {outcast, Jid};
 		       _ -> throw({error, "Invalid 'affiliation'"})
 		   end,
     try jid:decode(JidS) of
@@ -1755,7 +1786,7 @@ parse_affiliation_string(String) ->
     end.
 
 parse_subscription_string(String) ->
-    case str:tokens(String, <<":">>) of
+    case str:tokens(String, <<"=:">>) of
 	[_] ->
 	    throw({error, "Invalid 'subscribers' - missing nick"});
 	[_, _] ->
@@ -1860,6 +1891,7 @@ change_option(Option, Value, Config) ->
 	anonymous -> Config#config{anonymous = Value};
 	captcha_protected -> Config#config{captcha_protected = Value};
 	description -> Config#config{description = Value};
+	enable_hats -> Config#config{enable_hats = Value};
 	lang -> Config#config{lang = Value};
 	logging -> Config#config{logging = Value};
 	mam -> Config#config{mam = Value};
