@@ -5,7 +5,7 @@
 %%% Created : 16 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2025   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -71,8 +71,8 @@ start(Host, Opts) ->
     Mod = gen_mod:db_mod(Opts, ?MODULE),
     Mod:init(Host, Opts),
     init_cache(Mod, Host, Opts),
-    ejabberd_commands:register_commands(?MODULE, get_commands_spec()),
-    {ok, [{hook, remove_user, remove_user, 50},
+    {ok, [{commands, get_commands_spec()},
+          {hook, remove_user, remove_user, 50},
           {hook, disco_sm_features, get_sm_features, 50},
           {hook, pubsub_publish_item, pubsub_publish_item, 50},
           {hook, pubsub_delete_item, pubsub_delete_item, 50},
@@ -81,13 +81,8 @@ start(Host, Opts) ->
           {hook, webadmin_page_hostuser, webadmin_page_hostuser, 50},
           {iq_handler, ejabberd_sm, ?NS_PRIVATE, process_sm_iq}]}.
 
-stop(Host) ->
-    case gen_mod:is_loaded_elsewhere(Host, ?MODULE) of
-	false ->
-	    ejabberd_commands:unregister_commands(get_commands_spec());
-	true ->
-	    ok
-    end.
+stop(_Host) ->
+    ok.
 
 reload(Host, NewOpts, OldOpts) ->
     NewMod = gen_mod:db_mod(NewOpts, ?MODULE),
@@ -340,8 +335,8 @@ publish_pep_native_bookmarks(JID, Data) ->
 				    #bookmark_storage{conference = C} -> C;
 				    _ -> []
 				catch _:{xmpp_codec, Why} ->
-					  ?WARNING_MSG("Failed to decode bookmarks of ~ts: ~ts",
-						       [jid:encode(JID), xmpp:format_error(Why)]),
+					  ?DEBUG("Failed to decode bookmarks of ~ts: ~ts",
+						 [jid:encode(JID), xmpp:format_error(Why)]),
 					  []
 				end,
 		    PubOpts = [{persist_items, true}, {access_model, whitelist}, {max_items, max}, {notify_retract,true}, {notify_delete,true}, {send_last_published_item, never}],
@@ -447,17 +442,22 @@ pubsub_delete_item(_, _, _, _, _) ->
 
 -spec pubsub_item_to_storage_bookmark(#pubsub_item{}) -> {true, bookmark_conference()} | false.
 pubsub_item_to_storage_bookmark(#pubsub_item{itemid = {Id, _}, payload = [#xmlel{} = B | _]}) ->
-    case xmpp:decode(B) of
-	#pep_bookmarks_conference{name = Name, autojoin = AutoJoin, nick = Nick,
-				  password = Password} ->
-	    try jid:decode(Id) of
-		#jid{} = Jid ->
-		    {true, #bookmark_conference{jid = Jid, name = Name, autojoin = AutoJoin, nick = Nick,
-						password = Password}}
-	    catch _:_ ->
-		false
-	    end;
-	_ ->
+    try {xmpp:decode(B), jid:decode(Id)} of
+	{#pep_bookmarks_conference{name = Name, autojoin = AutoJoin,
+				   nick = Nick, password = Password},
+	 #jid{} = Jid} ->
+	    {true, #bookmark_conference{jid = Jid, name = Name,
+					autojoin = AutoJoin, nick = Nick,
+					password = Password}};
+	{_, _} ->
+	    false
+    catch
+	_:{xmpp_codec, Why} ->
+	    ?DEBUG("Failed to decode bookmark element (~ts): ~ts",
+		   [Id, xmpp:format_error(Why)]),
+	    false;
+	_:{bad_jid, _} ->
+	    ?DEBUG("Failed to decode bookmark ID (~ts)", [Id]),
 	    false
     end;
 pubsub_item_to_storage_bookmark(_) ->
@@ -487,15 +487,19 @@ storage_bookmark_to_xmpp_bookmark(#bookmark_conference{name = Name, autojoin = A
 
 -spec pubsub_item_to_map(#pubsub_item{}, map()) -> map().
 pubsub_item_to_map(#pubsub_item{itemid = {Id, _}, payload = [#xmlel{} = B | _]}, Map) ->
-    case xmpp:decode(B) of
-	#pep_bookmarks_conference{} = B2 ->
-	    try jid:decode(Id) of
-		#jid{} = Jid ->
-		    maps:put(jid:tolower(Jid), B2#pep_bookmarks_conference{extensions = undefined}, Map)
-	    catch _:_ ->
-		Map
-	    end;
-	_ ->
+    try {xmpp:decode(B), jid:decode(Id)} of
+	{#pep_bookmarks_conference{} = B1, #jid{} = Jid} ->
+	    B2 = B1#pep_bookmarks_conference{extensions = undefined},
+	    maps:put(jid:tolower(Jid), B2, Map);
+	{_, _} ->
+	    Map
+    catch
+	_:{xmpp_codec, Why} ->
+	    ?DEBUG("Failed to decode bookmark element (~ts): ~ts",
+		   [Id, xmpp:format_error(Why)]),
+	    Map;
+	_:{bad_jid, _} ->
+	    ?DEBUG("Failed to decode bookmark ID (~ts)", [Id]),
 	    Map
     end;
 pubsub_item_to_map(_, Map) ->
