@@ -3,7 +3,7 @@
 %%% Created :  2 Jun 2013 by Evgeniy Khramtsov <ekhramtsov@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2024   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2025   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -60,6 +60,11 @@ init_per_suite(Config) ->
     NewConfig.
 
 start_ejabberd(_) ->
+    TestBeams = case filelib:is_dir("../../test/") of
+       true -> "../../test/";
+       _ -> "../../lib/ejabberd/test/"
+    end,
+    application:set_env(ejabberd, external_beams, TestBeams),
     {ok, _} = application:ensure_all_started(ejabberd, transient).
 
 end_per_suite(_Config) ->
@@ -100,6 +105,7 @@ do_init_per_group(mysql, Config) ->
         {selected, _, _} ->
             mod_muc:shutdown_rooms(?MYSQL_VHOST),
             update_sql(?MYSQL_VHOST, Config),
+            stop_temporary_modules(?MYSQL_VHOST),
             set_opt(server, ?MYSQL_VHOST, Config);
         Err ->
             {skip, {mysql_not_available, Err}}
@@ -109,6 +115,7 @@ do_init_per_group(mssql, Config) ->
         {selected, _, _} ->
             mod_muc:shutdown_rooms(?MSSQL_VHOST),
             update_sql(?MSSQL_VHOST, Config),
+            stop_temporary_modules(?MSSQL_VHOST),
             set_opt(server, ?MSSQL_VHOST, Config);
         Err ->
             {skip, {mssql_not_available, Err}}
@@ -118,6 +125,7 @@ do_init_per_group(pgsql, Config) ->
         {selected, _, _} ->
             mod_muc:shutdown_rooms(?PGSQL_VHOST),
             update_sql(?PGSQL_VHOST, Config),
+            stop_temporary_modules(?PGSQL_VHOST),
             set_opt(server, ?PGSQL_VHOST, Config);
         Err ->
             {skip, {pgsql_not_available, Err}}
@@ -161,6 +169,10 @@ do_init_per_group(GroupName, Config) ->
 	_ -> NewConfig
     end.
 
+stop_temporary_modules(Host) ->
+    Modules = [mod_shared_roster],
+    [gen_mod:stop_module(Host, M) || M <- Modules].
+
 end_per_group(mnesia, _Config) ->
     ok;
 end_per_group(redis, _Config) ->
@@ -391,6 +403,7 @@ no_db_tests() ->
      auth_external_wrong_server,
      auth_external_invalid_cert,
      commands_tests:single_cases(),
+     configtest_tests:single_cases(),
      jidprep_tests:single_cases(),
      sm_tests:single_cases(),
      sm_tests:master_slave_cases(),
@@ -1094,7 +1107,17 @@ clear_table_queries(Queries) ->
       fun(Query, Acc) ->
               case split(str:to_lower(Query)) of
                   [<<"create">>, <<"table">>, Table|_] ->
-                      [<<"DELETE FROM ", Table/binary, ";">>|Acc];
+                      GlobalRamTables = [<<"bosh">>,
+                                         <<"oauth_client">>,
+                                         <<"oauth_token">>,
+                                         <<"proxy65">>,
+                                         <<"route">>],
+                      case lists:member(Table, GlobalRamTables) of
+                          true ->
+                              Acc;
+                          false ->
+                              [<<"DELETE FROM ", Table/binary, ";">>|Acc]
+                      end;
                   _ ->
                       Acc
               end
